@@ -28,6 +28,7 @@
 #include "gmag-cursor.h"
 #include "gmag-graphical-server.h"
 #include "GNOME_Magnifier.h"
+#include "magnifier-server.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +37,9 @@
 #include <libbonobo.h>
 #include <login-helper/login-helper.h>
 
-#include <gdk-pixbuf/gdk-pixbuf-io.h>
+#include <dbus/dbus-glib-bindings.h>
+
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
@@ -84,19 +87,20 @@ enum {
 typedef struct
 {
 	GNOME_Magnifier_RectBounds rectbounds;
+	RectBounds dbus_rectbounds;
 	GNOME_Magnifier_RectBounds viewport;
-	gboolean is_managed;
-	gint scroll_policy;
-	gfloat contrast;
-	gfloat zx;
-	gfloat zy;
-	gint32 xalign;
-	gint32 yalign;
-	guint32 border_color;
-	gint32 border_size;
-	gchar *smoothing_type;
-	gboolean inverse;
-
+	RectBounds dbus_viewport;
+	gboolean   is_managed;
+	gint       scroll_policy;
+	gfloat     contrast;
+	gfloat     zx;
+	gfloat     zy;
+	gint32     xalign;
+	gint32     yalign;
+	guint32    border_color;
+	gint32     border_size;
+	gchar     *smoothing_type;
+	gboolean   inverse;
 } MagnifierZoomRegionSaveProps;
 
 #ifdef DEBUG_CLIENT_CALLS
@@ -128,6 +132,8 @@ static void magnifier_init_window (Magnifier *magnifier, GdkScreen *screen);
 static void magnifier_adjust_source_size (Magnifier *magnifier);
 static gboolean _is_override_redirect = FALSE;
 
+static GHashTable *zoom_hash = NULL;
+
 static Window*
 mag_login_helper_get_raise_windows (LoginHelper *helper)
 {
@@ -138,7 +144,7 @@ mag_login_helper_get_raise_windows (LoginHelper *helper)
     if (magnifier && magnifier->priv && magnifier->priv->w)
     {
 	mainwin = g_new0 (Window, 2);
-	mainwin[0] = GDK_WINDOW_XWINDOW (magnifier->priv->w->window);
+	mainwin[0] = GDK_WINDOW_XWINDOW (gtk_widget_get_window (magnifier->priv->w));
 	mainwin[1] = None;
     }
     return mainwin;
@@ -149,7 +155,6 @@ mag_login_helper_get_device_reqs (LoginHelper *helper)
 {
     /* means "don't grab the xserver or core pointer", 
        and "we need to raise windows" */
-
     return LOGIN_HELPER_GUI_EVENTS | 
 	LOGIN_HELPER_POST_WINDOWS | 
 	LOGIN_HELPER_CORE_POINTER;
@@ -277,7 +282,7 @@ magnifier_set_cursor_from_pixbuf (Magnifier *magnifier,
 	GdkPixmap *pixmap, *mask;
 	gint width, height;
 	GdkGC *gc;
-	GdkDrawable *drawable = magnifier->priv->w->window;
+	GdkDrawable *drawable = gtk_widget_get_window (magnifier->priv->w);
 
 	if (magnifier->priv->cursor) {
 		g_object_unref (magnifier->priv->cursor);
@@ -304,8 +309,8 @@ magnifier_set_cursor_from_pixbuf (Magnifier *magnifier,
 		magnifier->priv->cursor = pixmap;
 		magnifier->priv->cursor_mask = mask;
 
-		xhot_string = g_object_get_data (cursor_pixbuf, "x_hot");
-		yhot_string = g_object_get_data (cursor_pixbuf, "y_hot");
+		xhot_string = g_object_get_data (G_OBJECT(cursor_pixbuf), "x_hot");
+		yhot_string = g_object_get_data (G_OBJECT(cursor_pixbuf), "y_hot");
 
 		if (xhot_string)
 			magnifier->cursor_hotspot.x = atoi (xhot_string);
@@ -484,8 +489,8 @@ magnifier_adjust_source_size (Magnifier *magnifier)
 				  magnifier->target_bounds.y2) /
 		(double) rect_bounds.y2;
 
-	/* we make our 'source' rectangle the largest available subsection which we aren't occupying */
 	if (magnifier_full_screen_capable (magnifier)) {
+	/* we make our 'source' rectangle the largest available subsection which we aren't occupying */
 		magnifier->source_bounds = rect_bounds;
 	} else if (MAX (hfract_left, hfract_right) >
 		   MAX (vfract_top, vfract_bottom)) {  /* vertical split,
@@ -534,6 +539,17 @@ magnifier_unref_zoom_region (gpointer data, gpointer user_data)
 	    Bonobo_Unknown_unref (zoom_region, &ev);
 }
 
+static void
+magnifier_dbus_unref_zoom_region (gpointer data, gpointer user_data)
+{
+/*	Magnifier *magnifier = user_data; NOT USED */
+	ZoomRegion *zoom_region = data;
+	
+	DBG(g_message ("unreffing zoom region"));
+
+	impl_dbus_zoom_region_dispose (zoom_region);
+}
+
 static GSList*
 magnifier_zoom_regions_save (Magnifier *magnifier)
 {
@@ -582,6 +598,7 @@ magnifier_zoom_regions_save (Magnifier *magnifier)
     }   
 
     magnifier->zoom_regions = NULL;
+    magnifier->zoom_regions_dbus = NULL;
 
     return save_props;
 }
@@ -773,7 +790,7 @@ magnifier_transform_cursor (Magnifier *magnifier)
 		g_object_unref (cursor_pixmap);
 		g_object_unref (mask_pixmap);
 		magnifier->priv->cursor = gdk_pixmap_new (
-			magnifier->priv->w->window,
+			gtk_widget_get_window (magnifier->priv->w),
 			size_x, size_y,
 			-1);
 		if (!GDK_IS_DRAWABLE (magnifier->priv->cursor)) 
@@ -782,7 +799,7 @@ magnifier_transform_cursor (Magnifier *magnifier)
 		    return;
 		}
 		magnifier->priv->cursor_mask = gdk_pixmap_new (
-			magnifier->priv->w->window,
+		        gtk_widget_get_window (magnifier->priv->w),
 			size_x, size_y,
 			1);
 		if (GDK_IS_DRAWABLE (magnifier->priv->cursor)) {
@@ -876,11 +893,19 @@ magnifier_reset_struts_at_idle (gpointer data)
 	if (data)
 	{
 		Magnifier *magnifier = MAGNIFIER (data);
+#if GTK_CHECK_VERSION (2,19,5)
+		if (magnifier->priv &&
+		    gtk_widget_get_realized (magnifier->priv->w) &&
+		    gmag_gs_check_set_struts (magnifier)) {
+			return FALSE;
+		}
+#else
 		if (magnifier->priv &&
 		    GTK_WIDGET_REALIZED (magnifier->priv->w) && 
 		    gmag_gs_check_set_struts (magnifier)) {
 			return FALSE;
 		}
+#endif
 	}
 	return TRUE;
 }
@@ -959,6 +984,387 @@ magnifier_get_property (BonoboPropertyBag *bag,
 	};
 }
 
+gboolean
+impl_dbus_magnifier_set_source_display (Magnifier *magnifier, gchar *source_display)
+{
+	gchar *full_display_string = source_display;
+	if (can_open_display (full_display_string))
+	{
+	    GSList *zoom_region_params = NULL;
+	    magnifier->source_screen_num =
+		magnifier_parse_display_name (magnifier,
+					      full_display_string,
+					      NULL);
+	    magnifier->source_display =
+		gdk_display_open (full_display_string);
+	    magnifier->source_display_name = g_strdup (full_display_string);
+	    zoom_region_params = magnifier_zoom_regions_save (magnifier);
+	    magnifier->priv->root =
+		gdk_screen_get_root_window (
+		    gdk_display_get_screen (
+			magnifier->source_display,
+			magnifier->source_screen_num));
+
+	    /* remove the source_drawable, since in the new display the
+	     * composite can be unavailable and the source_drawable is
+	     * where the desktop screen is composed. If this is not
+	     * freed it will be used even if the display doesn't use
+	     * composite what will lead to wrong behavior and crashes.
+	     * If composite is used in the new display this variable is
+	     * re-created */
+	    if (magnifier->priv->source_drawable) {
+		    g_object_unref (magnifier->priv->source_drawable);
+		    magnifier->priv->source_drawable = NULL;
+	    }
+
+	    /* attach listeners for DAMAGE, "dirty region", XFIXES
+	     * cursor changes */
+            magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
+	    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
+	    magnifier_adjust_source_size (magnifier);
+	    magnifier_zoom_regions_restore (magnifier, zoom_region_params);
+	    magnifier_warp_cursor_to_screen (magnifier);
+	    gmag_gs_check_set_struts (magnifier);
+	    gmag_gs_reset_overlay_at_idle (magnifier);
+	}
+	DBG(fprintf (stderr, "Set source display: \t%s\n", full_display_string));
+	
+	return TRUE;
+}
+
+gchar*
+impl_dbus_magnifier_get_source_display (Magnifier *magnifier)
+{
+	return g_strdup (magnifier->source_display_name);
+}
+
+gboolean
+impl_dbus_magnifier_set_target_display (Magnifier *magnifier, gchar *target_display)
+{
+	gchar *full_display_string = target_display;
+	if (can_open_display (full_display_string))
+	{
+	    magnifier->target_screen_num =
+		magnifier_parse_display_name (magnifier,
+					      full_display_string,
+					      NULL);
+	    magnifier->target_display =
+		gdk_display_open (full_display_string);
+	    magnifier->target_display_name = g_strdup (full_display_string);
+	    if (GTK_IS_WINDOW (magnifier->priv->w)) 
+	    {
+#ifdef REPARENT_GTK_WINDOW_WORKS
+		gtk_window_set_screen (GTK_WINDOW (magnifier->priv->w), 
+				       gdk_display_get_screen (
+					   magnifier->target_display,
+					   magnifier->target_screen_num));
+#else
+		GSList *zoom_region_params = NULL;
+		/* disconnect from the old window's destroy signal */
+		g_object_disconnect (magnifier->priv->w,
+			  "any_signal::realize", magnifier_realize, NULL,
+			  "any_signal::size_allocate", magnifier_size_allocate, NULL,
+			  "any_signal::destroy", magnifier_exit, NULL,
+		          NULL);
+		/* save the old zoom region state */
+	        zoom_region_params = magnifier_zoom_regions_save (magnifier);
+		/* destroy the old window */
+		gtk_widget_destroy (magnifier->priv->w);
+		/* and re-initialize... */
+		magnifier_init_window (magnifier, gdk_display_get_screen (
+					   magnifier->target_display,
+					   magnifier->target_screen_num));
+		/* restore the zoom regions in their new host magnifier window */
+		magnifier_zoom_regions_restore (magnifier, zoom_region_params);
+#endif
+	    }
+	    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
+	    magnifier_init_cursor_set (magnifier, magnifier->cursor_set); /* needed to reset pixmaps */
+	    if (magnifier->priv->overlay)
+		    gdk_window_move (magnifier->priv->overlay,
+				     magnifier->target_bounds.x1,
+				     magnifier->target_bounds.y1);
+	    else
+		    gtk_window_move (GTK_WINDOW (magnifier->priv->w),
+				     magnifier->target_bounds.x1,
+				     magnifier->target_bounds.y1);
+	    
+	    if ((magnifier->target_bounds.x2 - magnifier->target_bounds.x1 > 0) &&
+		(magnifier->target_bounds.y2 - magnifier->target_bounds.y1) > 0)
+	    {
+		    if (magnifier->priv->overlay)
+			    gdk_window_resize (
+				    magnifier->priv->overlay,
+				    magnifier->target_bounds.x2 -
+				    magnifier->target_bounds.x1,
+				    magnifier->target_bounds.y2 -
+				    magnifier->target_bounds.y1);
+		gtk_window_resize (GTK_WINDOW (magnifier->priv->w),
+			       magnifier->target_bounds.x2 - magnifier->target_bounds.x1,
+			       magnifier->target_bounds.y2 - magnifier->target_bounds.y1);
+	    DBG(fprintf (stderr, "Set target size: \t%d,%d to %d,%d\n", 
+		     magnifier->target_bounds.x1, magnifier->target_bounds.y1, magnifier->target_bounds.x2, magnifier->target_bounds.y2));
+	    }
+	    /* N. B. we don't reset the target bounds to the limits of the new display, because */
+	    /* doing so would override the client-specified magnifier size */
+	    /* magnifier_get_display_rect_bounds (magnifier, &magnifier->target_bounds, TRUE); */
+	    gmag_gs_check_set_struts (magnifier);
+	}
+	DBG(fprintf (stderr, "Set target display: \t%s (screen %d)\n", 
+		      full_display_string, magnifier->target_screen_num));
+		      
+	return TRUE;
+}
+
+gchar*
+impl_dbus_magnifier_get_target_display (Magnifier *magnifier)
+{
+	return g_strdup (magnifier->target_display_name);
+}
+
+gboolean
+impl_dbus_magnifier_set_source_size (Magnifier *magnifier, gint32 **bounds)
+{
+	magnifier->source_bounds.x1 = (*bounds)[0];
+	magnifier->source_bounds.y1 = (*bounds)[1];
+	magnifier->source_bounds.x2 = (*bounds)[2];
+	magnifier->source_bounds.y2 = (*bounds)[3];
+	DBG (fprintf (stderr, "Set source size: \t%d,%d to %d,%d\n", 
+		      magnifier->source_bounds.x1, magnifier->source_bounds.y1, magnifier->source_bounds.x2, magnifier->source_bounds.y2));
+		      
+	return TRUE;
+}
+
+GArray*
+impl_dbus_magnifier_get_source_size (Magnifier *magnifier)
+{
+	GArray *ret = g_array_new (FALSE, FALSE, sizeof (gint32));
+	
+	g_array_append_val (ret, magnifier->source_bounds.x1);
+	g_array_append_val (ret, magnifier->source_bounds.y1);
+	g_array_append_val (ret, magnifier->source_bounds.x2);
+	g_array_append_val (ret, magnifier->source_bounds.y2);
+	
+	return ret;
+}
+
+gboolean
+impl_dbus_magnifier_set_target_size (Magnifier *magnifier, gint32 **bounds)
+{
+	magnifier->target_bounds.x1 = (*bounds)[0];
+	magnifier->target_bounds.y1 = (*bounds)[1];
+	magnifier->target_bounds.x2 = (*bounds)[2];
+	magnifier->target_bounds.y2 = (*bounds)[3];
+
+	if (magnifier->priv->overlay)
+		gdk_window_move_resize (magnifier->priv->overlay,
+					magnifier->target_bounds.x1,
+					magnifier->target_bounds.y1,
+					magnifier->target_bounds.x2 -
+					magnifier->target_bounds.x1,
+					magnifier->target_bounds.y2 -
+					magnifier->target_bounds.y1);
+	else
+		gtk_window_move (GTK_WINDOW (magnifier->priv->w),
+				 magnifier->target_bounds.x1,
+				 magnifier->target_bounds.y1);
+	
+	gtk_window_resize (GTK_WINDOW (magnifier->priv->w),
+			   magnifier->target_bounds.x2 - magnifier->target_bounds.x1,
+			   magnifier->target_bounds.y2 - magnifier->target_bounds.y1);
+	gmag_gs_check_set_struts (magnifier);
+	DBG(fprintf (stderr, "Set target size: \t%d,%d to %d,%d\n", 
+		      magnifier->target_bounds.x1, magnifier->target_bounds.y1, magnifier->target_bounds.x2, magnifier->target_bounds.y2));
+	if (!strcmp (magnifier->target_display_name, magnifier->source_display_name) && 
+	    (magnifier->target_screen_num == magnifier->source_screen_num)) 
+	    magnifier_adjust_source_size (magnifier);
+	    
+	return TRUE;
+}
+
+GArray*
+impl_dbus_magnifier_get_target_size (Magnifier *magnifier)
+{
+	GArray *ret = g_array_new (FALSE, FALSE, sizeof (gint32));
+	
+	g_array_append_val (ret, magnifier->target_bounds.x1);
+	g_array_append_val (ret, magnifier->target_bounds.y1);
+	g_array_append_val (ret, magnifier->target_bounds.x2);
+	g_array_append_val (ret, magnifier->target_bounds.y2);
+	
+	return ret;
+}
+
+gboolean
+impl_dbus_magnifier_set_cursor_set (Magnifier *magnifier, gchar *cursor_set)
+{
+	magnifier_init_cursor_set (magnifier, g_strdup (cursor_set));
+	DBG (fprintf (stderr, "Setting cursor set: \t%s\n", cursor_set));
+	
+	return TRUE;
+}
+
+gchar*
+impl_dbus_magnifier_get_cursor_set (Magnifier *magnifier)
+{
+	return g_strdup (magnifier->cursor_set);
+}
+
+gboolean
+impl_dbus_magnifier_set_cursor_size (Magnifier *magnifier, gint x, gint y)
+{
+	magnifier->cursor_size_x = x;
+	magnifier->cursor_size_y = y;
+	magnifier_transform_cursor (magnifier);
+	DBG (fprintf (stderr, "Setting cursor size: \t%d\n", magnifier->cursor_size_x));
+	
+	return TRUE;
+}
+
+GArray*
+impl_dbus_magnifier_get_cursor_size (Magnifier *magnifier)
+{
+	GArray *ret = g_array_new (FALSE, FALSE, sizeof (gint));
+	
+	g_array_append_val (ret, magnifier->cursor_size_x);
+	g_array_append_val (ret, magnifier->cursor_size_y);
+	
+	return ret;
+}
+
+gboolean
+impl_dbus_magnifier_set_cursor_zoom (Magnifier *magnifier, double factor)
+{
+	magnifier->cursor_scale_factor = factor;
+	DBG (fprintf (stderr, "Setting cursor scale factor: \t%f\n", magnifier->cursor_scale_factor));
+	magnifier_transform_cursor (magnifier);
+	
+	return TRUE;
+}
+
+double
+impl_dbus_magnifier_get_cursor_zoom (Magnifier *magnifier)
+{
+	return magnifier->cursor_scale_factor;
+}
+
+gboolean
+impl_dbus_magnifier_set_cursor_color (Magnifier *magnifier, guint32 color)
+{
+	magnifier->cursor_color = color;
+	magnifier_transform_cursor (magnifier);
+	DBG (fprintf (stderr, "Setting cursor color: \t%u\n", (unsigned) magnifier->cursor_color));
+	
+	return TRUE;
+}
+
+guint32
+impl_dbus_magnifier_get_cursor_color (Magnifier *magnifier)
+{
+	return magnifier->cursor_color;
+}
+
+gboolean
+impl_dbus_magnifier_set_cursor_hotspot (Magnifier *magnifier, gint32 x, gint32 y)
+{
+	magnifier->cursor_hotspot.x = x;
+	magnifier->cursor_hotspot.y = y;
+	magnifier_transform_cursor (magnifier);
+	
+	return TRUE;
+}
+
+GArray*
+impl_dbus_magnifier_get_cursor_hotspot (Magnifier *magnifier)
+{
+	GArray *ret = g_array_new (FALSE, FALSE, sizeof (gint32));
+	
+	g_array_append_val (ret, magnifier->cursor_hotspot.x);
+	g_array_append_val (ret, magnifier->cursor_hotspot.y);
+	
+	return ret;
+}
+
+gint32
+impl_dbus_magnifier_get_cursor_default_size (Magnifier *magnifier)
+{
+	gint32 csize;
+	
+	if (magnifier->priv->cursor) {
+		gdk_drawable_get_size (magnifier->priv->cursor, &csize, &csize);
+	}
+	
+	return csize;
+}
+
+gboolean
+impl_dbus_magnifier_set_crosswire_size (Magnifier *magnifier, gint size)
+{
+	magnifier->crosswire_size = size;
+	DBG (fprintf (stderr, "Setting crosswire size: \t%d\n", magnifier->crosswire_size));
+	magnifier_zoom_regions_update_pointer (magnifier);
+	
+	return TRUE;
+}
+
+gint
+impl_dbus_magnifier_get_crosswire_size (Magnifier *magnifier)
+{
+	return magnifier->crosswire_size;
+}
+
+gboolean
+impl_dbus_magnifier_set_crosswire_length (Magnifier *magnifier, gint length)
+{
+	GNOME_Magnifier_RectBounds rect_bounds;
+	rect_bounds.x1 = 0;
+	rect_bounds.y1 = 0;
+	rect_bounds.x2 = 4096;
+	rect_bounds.y2 = 4096;
+
+	magnifier->crosswire_length = length;
+	DBG (fprintf (stderr, "Setting crosswire length: \t%d\n", magnifier->crosswire_length));
+	magnifier_zoom_regions_mark_dirty (magnifier, rect_bounds);
+	
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_set_crosswire_clip (Magnifier *magnifier, gboolean clip)
+{
+	magnifier->crosswire_clip = clip;
+	DBG (fprintf (stderr, "Setting crosswire clip: \t%s\n", magnifier->crosswire_clip ? "true" : "false"));
+	magnifier_zoom_regions_update_pointer (magnifier);
+	
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_get_crosswire_clip (Magnifier *magnifier)
+{
+	return magnifier->crosswire_clip;
+}
+
+gint
+impl_dbus_magnifier_get_crosswire_length (Magnifier *magnifier) {
+	return magnifier->crosswire_length;
+}
+
+gboolean
+impl_dbus_magnifier_set_crosswire_color (Magnifier *magnifier, guint32 color)
+{
+	magnifier->crosswire_color = color;
+	DBG (fprintf (stderr, "Setting crosswire color: \t%ld\n", (long) magnifier->crosswire_color));
+	
+	return TRUE;
+}
+
+guint32
+impl_dbus_magnifier_get_crosswire_color (Magnifier *magnifier)
+{
+	return magnifier->crosswire_color;
+}
+
 static void
 magnifier_set_property (BonoboPropertyBag *bag,
 			BonoboArg *arg,
@@ -1199,9 +1605,12 @@ magnifier_do_dispose (Magnifier *magnifier)
 	bonobo_activation_active_server_unregister (
 		MAGNIFIER_OAFIID, BONOBO_OBJREF (magnifier));
 
+	if (magnifier->zoom_regions_dbus)
+		g_list_free (magnifier->zoom_regions_dbus);
 	if (magnifier->zoom_regions)
 		g_list_free (magnifier->zoom_regions);
 	magnifier->zoom_regions = NULL;
+	magnifier->zoom_regions_dbus = NULL;
 	
 	bonobo_main_quit ();
 }
@@ -1214,9 +1623,10 @@ magnifier_gobject_dispose (GObject *object)
 	BONOBO_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
-static
+static inline
 CORBA_boolean
-impl_magnifier_support_colorblind_filters (Magnifier *magnifier)
+impl_magnifier_support_colorblind_filters (PortableServer_Servant _servant G_GNUC_UNUSED,
+					   CORBA_Environment *ev G_GNUC_UNUSED)
 {
 #ifdef HAVE_COLORBLIND
 	return CORBA_TRUE;
@@ -1258,6 +1668,139 @@ impl_magnifier_full_screen_capable (PortableServer_Servant servant,
 	return CORBA_FALSE;
 }
 
+gboolean
+impl_dbus_magnifier_full_screen_capable (Magnifier *magnifier)
+{
+	if ((strcmp (magnifier->source_display_name,
+		     magnifier->target_display_name) != 0) ||
+		gmag_gs_use_compositor (magnifier)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+impl_dbus_magnifier_hide_cursor (Magnifier *magnifier)
+{
+	gmag_gs_hide_cursor (magnifier);
+	
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_show_cursor (Magnifier *magnifier)
+{
+	gmag_gs_show_cursor (magnifier);
+
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_support_colorblind_filters (Magnifier *magnifier)
+{
+#ifdef HAVE_COLORBLIND
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
+gchar*
+impl_dbus_magnifier_create_zoom_region (Magnifier *magnifier,
+					const double zx,
+				   	const double zy,
+				   	const gint32 **roi,
+				   	const gint32 **viewport)
+{
+	ZoomRegion *zoom_region = zoom_region_new();
+
+	DBG (fprintf (stderr, "Create zoom region: \tzoom %f,%f, viewport %d,%d to %d,%d\n", (float) zx, (float) zy, (*viewport)[0]/*x1*/, (*viewport)[1]/*y1*/, (*viewport)[2]/*x2*/, (*viewport)[3]/*y3*/));
+
+	/* FIXME:
+	 * shouldn't do this here, since it causes the region to get
+	 * mapped onto the parent, if if it's not explicitly added!
+	 */
+	DBG(g_message ("creating zoom region with parent %p", magnifier));
+	zoom_region->priv->parent = magnifier;
+
+	impl_dbus_zoom_region_set_mag_factor (zoom_region, zx, zy);
+	
+	impl_dbus_zoom_region_move_resize (zoom_region, viewport);
+
+	impl_dbus_zoom_region_set_roi (zoom_region, roi);
+
+	gtk_widget_set_size_request (magnifier->priv->canvas,
+				     (*viewport)[2] - (*viewport)[0],
+				     (*viewport)[3] - (*viewport)[1]);
+	gtk_widget_show (magnifier->priv->canvas);
+	gtk_widget_show (magnifier->priv->w);
+	
+	g_hash_table_insert (zoom_hash, g_strdup(zoom_region->object_path), zoom_region);
+
+	return g_strdup (zoom_region->object_path);
+}
+
+gchar**
+impl_dbus_magnifier_get_zoom_regions (Magnifier *magnifier)
+{
+	ZoomRegion *zoom_region;
+	gchar **list;
+	int i, len;
+
+	len = g_list_length (magnifier->zoom_regions);
+	list = g_malloc0 (sizeof (gchar *) * len);
+	for (i = 0; i < len; ++i) {
+		zoom_region = g_list_nth_data (magnifier->zoom_regions_dbus, i);
+		list[i] = g_strdup (zoom_region->object_path);
+	}
+
+	DBG (fprintf (stderr, "Get zoom regions: \t%d\n", len));
+	
+	return list; 
+}
+
+gboolean
+impl_dbus_magnifier_add_zoom_region (Magnifier *magnifier, gchar *zoom_region_path)
+{
+	if (!magnifier->source_initialized) 
+	{
+		magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
+	}
+
+	/* FIXME: this needs proper lifecycle management */
+	ZoomRegion *zoom_region = g_hash_table_lookup (zoom_hash, zoom_region_path);
+	magnifier->zoom_regions = g_list_append (magnifier->zoom_regions, BONOBO_OBJREF (zoom_region));
+	magnifier->zoom_regions_dbus = g_list_append (magnifier->zoom_regions_dbus, zoom_region);
+	g_hash_table_remove (zoom_hash, zoom_region_path);
+	gmag_gs_check_set_struts (magnifier);
+
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_clear_all_zoom_regions (Magnifier *magnifier)
+{
+	g_list_foreach (magnifier->zoom_regions_dbus,
+			magnifier_dbus_unref_zoom_region, magnifier);
+	g_list_foreach (magnifier->zoom_regions,
+			magnifier_unref_zoom_region, magnifier);
+	g_list_free (magnifier->zoom_regions);
+	g_list_free (magnifier->zoom_regions_dbus);
+	magnifier->zoom_regions = NULL;
+	magnifier->zoom_regions_dbus = NULL;
+
+	return TRUE;
+}
+
+gboolean
+impl_dbus_magnifier_dispose (Magnifier *magnifier)
+{
+	magnifier_do_dispose (magnifier);
+	
+	return TRUE;
+}
+
 static void
 impl_magnifier_set_source_display (PortableServer_Servant servant,
 				   const CORBA_char *display,
@@ -1475,6 +2018,7 @@ impl_magnifier_clear_all_zoom_regions (PortableServer_Servant servant,
 			magnifier_unref_zoom_region, magnifier);
 	g_list_free (magnifier->zoom_regions);
 	magnifier->zoom_regions = NULL;
+	magnifier->zoom_regions_dbus = NULL;
 }
 
 static void
@@ -1497,8 +2041,7 @@ magnifier_class_init (MagnifierClass *klass)
 	epv->fullScreenCapable = impl_magnifier_full_screen_capable;
 	epv->hideCursor = impl_magnifier_hide_cursor;
 	epv->showCursor = impl_magnifier_show_cursor;
-	epv->supportColorblindFilters =
-		impl_magnifier_support_colorblind_filters;
+	epv->supportColorblindFilters = impl_magnifier_support_colorblind_filters;
         epv->_set_SourceDisplay = impl_magnifier_set_source_display;
 	epv->_set_TargetDisplay = impl_magnifier_set_target_display;
         epv->_get_SourceDisplay = impl_magnifier_get_source_display;
@@ -1526,7 +2069,6 @@ magnifier_properties_init (Magnifier *magnifier)
 			g_cclosure_new_object (
 				G_CALLBACK (magnifier_set_property),
 				G_OBJECT (magnifier)));
-
 	/* Aggregate so magnifier implements Bonobo_PropertyBag */
 	bonobo_object_add_interface (BONOBO_OBJECT (magnifier),
 				     BONOBO_OBJECT (magnifier->property_bag));
@@ -1680,7 +2222,8 @@ static void
 magnifier_init_window (Magnifier *magnifier, GdkScreen *screen)
 {
 	GtkWindowType mag_win_type = GTK_WINDOW_TOPLEVEL;
-	if (_is_override_redirect) mag_win_type = GTK_WINDOW_POPUP;
+	if (_is_override_redirect || gmag_gs_use_compositor (magnifier))
+		mag_win_type = GTK_WINDOW_POPUP;
 
 	magnifier->priv->w =
 		g_object_connect (gtk_widget_new (gtk_window_get_type (),
@@ -1709,6 +2252,7 @@ magnifier_init (Magnifier *magnifier)
 	magnifier->priv = g_new0 (MagnifierPrivate, 1);
 	magnifier_properties_init (magnifier);
 	magnifier->zoom_regions = NULL;
+	magnifier->zoom_regions_dbus = NULL;
 	magnifier->source_screen_num = 0;
 	magnifier->target_screen_num = 0;
 	magnifier->source_display_name = g_strdup (":0.0");
@@ -1745,6 +2289,8 @@ magnifier_init (Magnifier *magnifier)
 #ifdef DEBUG_CLIENT_CALLS
 	client_debug = (g_getenv ("MAG_CLIENT_DEBUG") != NULL);
 #endif
+
+	zoom_hash = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 GdkDrawable *
@@ -1781,6 +2327,10 @@ magnifier_new (gboolean override_redirect)
 {
 	Magnifier *mag;
 	MagLoginHelper *helper;
+	GError *error = NULL;
+	MagnifierClass *klass = NULL;
+	DBusGProxy *driver_proxy;
+	guint request_req;
 	int ret;
 
 	_is_override_redirect = override_redirect;
@@ -1805,10 +2355,33 @@ magnifier_new (gboolean override_redirect)
 	    } else
 		g_error ("Error registering magnifier server.\n");
 	}
+	
+	klass = MAGNIFIER_GET_CLASS (mag);
+	
+	klass->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	if (klass->connection == NULL) {
+		g_warning ("Unable to connect to dbus: %s", error->message);
+		g_error_free (error);
+		return NULL;
+	}
+	
+	dbus_g_object_type_install_info (MAGNIFIER_TYPE, &dbus_glib_impl_dbus_magnifier_object_info);
+	
+	dbus_g_connection_register_g_object (klass->connection, "/org/freedesktop/gnome/Magnifier",
+					     G_OBJECT (_this_magnifier));
+	
+	driver_proxy = dbus_g_proxy_new_for_name (klass->connection, DBUS_SERVICE_DBUS, DBUS_PATH_DBUS,
+						  DBUS_INTERFACE_DBUS);
+						  
+	if (!org_freedesktop_DBus_request_name (driver_proxy, "org.freedesktop.gnome.Magnifier", 0, &request_req,
+						&error)) {
+		g_warning ("Unable to register service: %s", error->message);
+		g_error_free (error);					
+	}
 
 	g_timeout_add (500, magnifier_reset_struts_at_idle, mag);
 	g_timeout_add (500, gmag_gs_reset_overlay_at_idle, mag);
-
+	
 	return mag;
 }
 
